# https://loguru.readthedocs.io/en/stable/overview.html#ready-to-use-out-of-the-box-without-boilerplate

import sys
from loguru import logger as context_logger


class GeneralLog:
    # __instance = None
    # def __new__(cls, *args, **kwargs):
    #     if not cls.__instance:
    #         cls.__instance = super(GeneralLog, cls).__new__(cls, *args, **kwargs)
    #     return cls.__instance

    # def __init__(self, *args, log_file_rec: bool = False, log_file_name: str = '', log_file_addr: str = '', **kwargs):
    def __init__(self, *args, **kwargs):
        super(GeneralLog, self).__init__(*args, **kwargs)
        log_file_rec = kwargs.get('log_file_rec')
        log_file_retention_days = kwargs.get('log_file_retention_days', 7)
        log_file_name = kwargs.get('log_file_name')
        log_file_addr = kwargs.get('log_file_addr')
        context_logger.remove()
        context_logger.add(sys.stdout, colorize=True, format="<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> <cyan>{extra[uuid1]}</cyan> <blue>{extra[uuid2]}</blue> <level>{message}</level>", enqueue=True)
        if log_file_rec:
            if log_file_name and log_file_addr:
                log_name = f'{log_file_addr}{log_file_name}.log'
                log_error_name = f'{log_file_addr}{log_file_name}_error.log'
            else:
                log_name = f'{log_file_name}.log'
                log_error_name = f'{log_file_name}_error.log'
            # https://loguru.readthedocs.io/en/stable/_modules/loguru/_logger.html?highlight=cyan#
            context_logger.add(log_name, format="<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> {level} <cyan>{extra[uuid1]}</cyan> <blue>{extra[uuid2]}</blue> {message}", rotation="23:00", retention=f"{log_file_retention_days} days", encoding='utf-8', enqueue=True)
            context_logger.add(log_error_name, format="<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> {level} <cyan>{extra[uuid1]}</cyan> <blue>{extra[uuid2]}</blue> {message}", rotation="23:00", retention=f"{log_file_retention_days} days", encoding='utf-8', enqueue=True, level='ERROR')

        self.context_logger = context_logger
        self.logger = self.get_logger

    def get_logger(self, **kwargs):
        uuid1 = kwargs.get('uuid1') or ''
        uuid2 = kwargs.get('uuid2') or ''
        self.context_logger = self.context_logger.patch(lambda record: record["extra"].update(uuid1=uuid1, uuid2=uuid2))
        return self.context_logger

# aa = GeneralLog()
# aa.logger.info('info')
# aa.logger.warning('warning')
# aa.logger.debug('debug')
# aa.logger.error('error')


# from loguru import logger
# import os
#
# logs_dir = f"{os.path.dirname(os.path.dirname(__file__))}/log"
# log_file_name = 'ApiTestLogs'
#
#
# class Loggings:
#     __instance = None
#     logger.add(f"{logs_dir}/{log_file_name}.log", rotation="500MB", encoding="utf-8", enqueue=True,
#                retention="10 days")
#
#     def __new__(cls, *args, **kwargs):
#         if not cls.__instance:
#             cls.__instance = super(Loggings, cls).__new__(cls, *args, **kwargs)
#
#         return cls.__instance
#     def __init__(self):
#         self.className = self.__class__.__name__
#
#     def info(self, msg):
#
#         return logger.info(msg)
#
#     def debug(self, msg):
#         return logger.debug(msg)
#
#     def warning(self, msg):
#         return logger.warning(msg)
#
#     def error(self, msg):
#         '''
#         打印错误信息
#         '''
#         return logger.error(msg)
#
#     def exception(self,msg):
#         '''
#         打印异常信息方法
#         '''
#         return logger.exception(msg)



import opencc


class GeneralTraditionalSimplifiedChinese:
    """
    hk2s: Traditional Chinese (Hong Kong standard) to Simplified Chinese

    s2hk: Simplified Chinese to Traditional Chinese (Hong Kong standard)

    s2t: Simplified Chinese to Traditional Chinese

    s2tw: Simplified Chinese to Traditional Chinese (Taiwan standard)

    s2twp: Simplified Chinese to Traditional Chinese (Taiwan standard, with phrases)

    t2hk: Traditional Chinese to Traditional Chinese (Hong Kong standard)

    t2s: Traditional Chinese to Simplified Chinese

    t2tw: Traditional Chinese to Traditional Chinese (Taiwan standard)

    tw2s: Traditional Chinese (Taiwan standard) to Simplified Chinese

    tw2sp: Traditional Chinese (Taiwan standard) to Simplified Chinese (with phrases)
    """

    def __init__(self, *args, **kwargs):
        super(GeneralTraditionalSimplifiedChinese, self).__init__(*args, **kwargs)

    @staticmethod
    def traditional_chinese_to_simplified(chinese_string: str) -> str:
        """
        t2s - 繁体转简体（Traditional Chinese to Simplified Chinese）
        :param chinese_string: str
        :return: str
        """
        cc = opencc.OpenCC('t2s')
        s = cc.convert(chinese_string)
        return s

    @staticmethod
    def simplified_chinese_to_traditional_chinese(chinese_string: str) -> str:
        """
        s2t - 简体转繁体（Simplified Chinese to Traditional Chinese）
        :param chinese_string: str
        :return: str
        """
        cc = opencc.OpenCC('s2t')
        s = cc.convert(chinese_string)
        return s


if __name__ == '__main__':
    print(GeneralTraditionalSimplifiedChinese().traditional_chinese_to_simplified('眾議長與李克強會談'))
    print(GeneralTraditionalSimplifiedChinese().simplified_chinese_to_traditional_chinese('众议长与李克强会谈'))



# 布隆过滤器,基于redis from：https://blog.csdn.net/bone_ace/article/details/53107018

from hashlib import md5

class SimpleBloomFilterHash:
    def __init__(self, cap, seed):
        self.cap = cap
        self.seed = seed

    def hash(self, value):
        ret = 0
        for i in range(len(value)):
            ret += self.seed * ret + ord(value[i])
        return (self.cap - 1) & ret


class AioGeneralBloomFilter:
    def __init__(self, *args, block_num=2, key='bloomfilter', **kwargs):
        # super(AioGeneralBloomFilter, self).__init__(*args, **kwargs)
        self.bloom_filter_timeout = kwargs.get('bloom_filter_timeout')
        self.bit_size = 1 << 31  # Redis的String类型最大容量为512M，现使用256M
        self.bloom_filter_seeds = [5, 7, 11, 13, 31, 37, 61]
        self.bloom_filter_key = key
        self.bloom_filter_block_num = block_num
        self.bloom_filter_hashfunc = []
        for seed in self.bloom_filter_seeds:
            self.bloom_filter_hashfunc.append(SimpleBloomFilterHash(self.bit_size, seed))

    async def aio_is_bloom_filter_contains(self, str_input:str)->int:
        try:
            if self.aio_redis_conn:
                if not str_input:
                    return False
                m5 = md5()
                str_input = str_input.encode('utf-8')
                m5.update(str_input)
                str_input = m5.hexdigest()
                ret = True
                name = self.bloom_filter_key + str(int(str_input[0:2], 16) % self.bloom_filter_block_num)
                for f in self.bloom_filter_hashfunc:
                    loc = f.hash(str_input)
                    ret = ret & await self.aio_redis_conn.getbit(name, loc)
                return ret
            else:
                raise Exception(f'需要事先建立redis链接（self.aio_redis_conn）')
        except Exception as e:
            print(e)

    async def aio_bloom_filter_insert(self, str_input:str, bloom_filter_timeout=None)->int:
        if self.aio_redis_conn:
            if bloom_filter_timeout:
                time_out_num = bloom_filter_timeout
            else:
                time_out_num = self.bloom_filter_timeout
            m5 = md5()
            if not str_input:
                return False
            str_input = str_input.encode('utf-8')
            m5.update(str_input)
            str_input = m5.hexdigest()
            name = self.bloom_filter_key + str(int(str_input[0:2], 16) % self.bloom_filter_block_num)
            for f in self.bloom_filter_hashfunc:
                loc = f.hash(str_input)
                if await self.aio_redis_conn.exists(name):  # 1 存在  0 不存在
                    await self.aio_redis_conn.setbit(name, loc, 1)
                else:
                    await self.aio_redis_conn.setbit(name, loc, 1)
                    await self.aio_redis_conn.expire(name, time_out_num)
            if await self.aio_redis_conn.ttl(name) == -2:  # key 不存在
                pass
            if await self.aio_redis_conn.ttl(name) == -1:  # 无过期时间
                await self.aio_redis_conn.delete(name)
            return 1
            # if await self.aio_redis_conn.ttl(name) < 300:  # 剩下五分钟过期直接删除key
            #     await self.aio_redis_conn.delete(name)
        else:
            raise Exception(f'需要事先建立redis链接（self.aio_redis_conn）')


# 工具箱中的工具都是开箱即用的，不依赖特别的数据、配置和业务逻辑

from sometools.string_tools.string_cleaning import GeneralString
from sometools.log_tools.logger_main import GeneralLog
from sometools.datetime_tools.date_conversion import GeneralDatetime
from sometools.chinese_to_pinyin_acronym.chinese_to_pinyin import GeneralChineseToPinyin
from sometools.traditional_simplified_chinese_conversion.traditional_simplified_chinese import \
    GeneralTraditionalSimplifiedChinese
from sometools.url_tools.url_encode_decode import GeneralUrlEncodeDecode
from sometools.redis_tools.redis import GeneralRedis


class Common_tools(GeneralChineseToPinyin, GeneralTraditionalSimplifiedChinese, GeneralUrlEncodeDecode, GeneralDatetime,
                   GeneralString, GeneralLog, GeneralRedis):
    def __init__(self, *args, **kwargs):
        super(Common_tools, self).__init__(*args, **kwargs)



from setuptools import setup
from setuptools import find_packages

with open("README.rst", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(name='SomeTools',
      version='0.1.22',
      description="Some python tools",
      author="zhangkun",
      author_email="zk.kyle@foxmail.com",
      project_urls={
          'Documentation': 'https://github.com/584807419/SomeTools',
          'Funding': 'https://github.com/584807419/SomeTools',
          'Source': 'https://github.com/584807419/SomeTools',
          'Tracker': 'https://github.com/584807419/SomeTools',
      },
      keywords=("Python", "Tools"),
      license='',
      long_description=long_description,  # 包的详细介绍，一般在README.md文件内
      long_description_content_type="text/markdown",
      url="https://pypi.org/project/SomeTools/",  # 自己项目地址，比如github的项目地址
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
      ],
      python_requires='>=3.6',  # 对python的最低版本要求
      packages=find_packages(),
      install_requires=[
          "datetime",
          "loguru",  # 高效优雅的日志显示
          "opencc-python-reimplemented",  # 繁体简体转换
          # "orjson",  # 底层使用了rust，Python下最快的json库,比 ujson 快 3 倍，比 json 快 6 倍
      ],
      py_modules=['sometools'],
      include_package_data=True,
      platforms="any",
      scripts=[],
      )

# https://blog.csdn.net/yifengchaoran/article/details/113447773
# python setup.py sdist bdist_wheel
# python -m twine upload --repository pypi dist/*
# pip install SomeTools -i https://pypi.python.org/simple



import asyncio
import aioredis
import platform

if not (platform.system() == 'Windows'):
    import uvloop

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())  # 使用 uvloop 来替换 asyncio 内部的事件循环。


class GeneralAsyncIoRedis:
    def __init__(self, *args, **kwargs):
        super(GeneralAsyncIoRedis, self).__init__(*args, **kwargs)

    async def get_async_redis_conn(self, **kwargs):
        return await aioredis.create_redis_pool((kwargs.get('redis_host'), kwargs.get('redis_port')), encoding='utf-8',
                                                db=kwargs.get('redis_db'), password=kwargs.get('redis_pwd'),
                                                loop=asyncio.get_event_loop())

    async def aio_redis_conn_close(self):
        if self.aio_redis_conn:
            self.aio_redis_conn.close()
            await self.aio_redis_conn.wait_closed()

# async def test():
#     redis = GeneralAsyncIoRedis()
#     r = await redis.get_redis_pool()
#     await redis.set('my-key', 'value')
#     value = await redis.get('my-key', encoding='utf-8')
#     await redis.close()



# URL为何要编码、解码？
#
#     通常如果一样东西需要编码，说明这样东西并不适合传输。原因多种多样，如Size过大，包含隐私数据。对于Url来说，之所以要进行编码，是因为Url中有些字符会引起歧义。
#
#     例如，Url参数字符串中使用key=value键值对这样的形式来传参，键值对之间以&符号分隔，如/s?q=abc&ie=utf-8。如果你的value字符串中包含了=或者&，那么势必会造成接收Url的服务器解析错误，因此必须将引起歧义的&和=符号进行转义，也就是对其进行编码。
#
#     又如，Url的编码格式采用的是ASCII码，而不是Unicode，这也就是说你不能在Url中包含任何非ASCII字符，例如中文。否则如果客户端浏览器和服务端浏览器支持的字符集不同的情况下，中文可能会造成问题。
# -*- coding: utf-8 -*-

from urllib.request import quote, unquote

import opencc


class GeneralUrlEncodeDecode:
    def __init__(self, *args, **kwargs):
        super(GeneralUrlEncodeDecode, self).__init__(*args, **kwargs)

    @staticmethod
    def url_encode(url: str, encoding: str = "utf-8") -> str:
        # 编码
        # utf8编码，指定安全字符
        ret = quote(url, safe=";/?:@&=+$,", encoding="utf-8")
        return ret
        # gbk编码
        # ret2 = quote(url1, encoding="gbk")

    @staticmethod
    def url_decode(url: str, encoding: str = "utf-8") -> str:
        # 编码
        # utf8编码，指定安全字符
        ret = unquote(url, encoding='utf-8')
        return ret


if __name__ == '__main__':
    print(GeneralUrlEncodeDecode().url_encode('https://www.baidu.com/s?wd=中国'))
    print(GeneralUrlEncodeDecode().url_decode('https://www.baidu.com/s?wd=%E4%B8%AD%E5%9B%BD'))


